@@ -62,11 +62,24 @@ kdump_is_vlan() {
      [ -f /proc/net/vlan/"$1" ]
 }
 
+# $1: netdev name
+source_ifcfg_file() {
+    local ifcfg_file
+
+    ifcfg_file=$(get_ifcfg_filename $1)
+    if [ -f "${ifcfg_file}" ]; then
+        . ${ifcfg_file}
+    else
+        dwarning "The ifcfg file of $1 is not found!"
+    fi
+}
+
 # $1: netdev name
 kdump_setup_dns() {
     local _nameserver _dns
     local _dnsfile=${initdir}/etc/cmdline.d/42dns.conf
-    . /etc/sysconfig/network-scripts/ifcfg-$1
+
+    source_ifcfg_file $1
 
     [ -n "$DNS1" ] && echo "nameserver=$DNS1" > "$_dnsfile"
     [ -n "$DNS2" ] && echo "nameserver=$DNS2" >> "$_dnsfile"
@@ -148,7 +161,11 @@ kdump_get_perm_addr() {
 kdump_setup_ifname() {
     local _ifname
 
-    if [[ $1 =~ eth* ]]; then
+    # If ifname already has 'kdump-' prefix, we must be switching from
+    # fadump to kdump. Skip prefixing 'kdump-' in this case as adding
+    # another prefix may truncate the ifname. Since an ifname with
+    # 'kdump-' is already persistent, this should be fine.
+    if [[ $1 =~ eth* ]] && [[ ! $1 =~ ^kdump-* ]]; then
         _ifname="kdump-$1"
     else
         _ifname="$1"
@@ -189,7 +206,9 @@ kdump_setup_bond() {
     done
     echo -n " bond=$_netdev:$(echo $_slaves | sed 's/,$//')" >> ${initdir}/etc/cmdline.d/42bond.conf
     # Get bond options specified in ifcfg
-    . /etc/sysconfig/network-scripts/ifcfg-$_netdev
+
+    source_ifcfg_file $_netdev
+
     bondoptions="$(echo :$BONDING_OPTS | sed 's/\s\+/,/')"
     echo "$bondoptions" >> ${initdir}/etc/cmdline.d/42bond.conf
 }
@@ -244,7 +263,9 @@ kdump_setup_vlan() {
 # $1: netdev name
 kdump_setup_znet() {
     local _options=""
-    . /etc/sysconfig/network-scripts/ifcfg-$1
+
+    source_ifcfg_file $1
+
     for i in $OPTIONS; do
         _options=${_options},$i
     done
@@ -336,7 +357,7 @@ kdump_install_net() {
     # the default gate way for network dump, eth1 in the fence kdump path will
     # call kdump_install_net again and we don't want eth1 to be the default
     # gateway.
-    if [ ! -f ${initdir}${initdir}/etc/cmdline.d/60kdumpnic.conf ] &&
+    if [ ! -f ${initdir}/etc/cmdline.d/60kdumpnic.conf ] &&
        [ ! -f ${initdir}/etc/cmdline.d/70bootdev.conf ]; then
         echo "kdumpnic=$(kdump_setup_ifname $_netdev)" > ${initdir}/etc/cmdline.d/60kdumpnic.conf
         echo "bootdev=$(kdump_setup_ifname $_netdev)" > ${initdir}/etc/cmdline.d/70bootdev.conf
@@ -433,6 +454,11 @@ kdump_install_conf() {
         ssh|nfs)
             kdump_install_net "$config_val"
             ;;
+        dracut_args)
+            if [[ $(get_dracut_args_fstype "$config_val") = nfs* ]] ; then
+                kdump_install_net "$(get_dracut_args_target "$config_val")"
+            fi
+            ;;
         kdump_pre|kdump_post|extra_bins)
             dracut_install $config_val
             ;;
@@ -494,7 +520,11 @@ kdump_get_iscsi_initiator() {
     return 1
 }
 
-# No ibft handling yet.
+# Figure out iBFT session according to session type
+is_ibft() {
+    [ "$(kdump_iscsi_get_rec_val $1 "node.discovery_type")" = fw ]
+}
+
 kdump_setup_iscsi_device() {
     local path=$1
     local tgt_name; local tgt_ipaddr;
@@ -517,6 +547,10 @@ kdump_setup_iscsi_device() {
         return 1
     fi
 
+    if is_ibft ${path}; then
+        return
+    fi
+
     tgt_name=$(kdump_iscsi_get_rec_val ${path} "node.name")
     tgt_ipaddr=$(kdump_iscsi_get_rec_val ${path} "node.conn\[0\].address")
 
@@ -712,8 +746,9 @@ install() {
     ln_r "$systemdsystemunitdir/kdump-capture.service" "$systemdsystemunitdir/initrd.target.wants/kdump-capture.service"
     inst "$moddir/kdump-error-handler.sh" "/usr/bin/kdump-error-handler.sh"
     inst "$moddir/kdump-error-handler.service" "$systemdsystemunitdir/kdump-error-handler.service"
-    # Replace existing emergency service
+    # Replace existing emergency service and emergency target
     cp "$moddir/kdump-emergency.service" "$initdir/$systemdsystemunitdir/emergency.service"
+    cp "$moddir/kdump-emergency.target" "$initdir/$systemdsystemunitdir/emergency.target"
     # Also redirect dracut-emergency to kdump error handler
     ln_r "$systemdsystemunitdir/emergency.service" "$systemdsystemunitdir/dracut-emergency.service"
 
@@ -722,12 +757,3 @@ install() {
     # at some point of time.
     kdump_check_iscsi_targets
 }
-
-installkernel() {
-    wdt=$(lsmod|cut -f1 -d' '|grep "wdt$")
-    if [ -n "$wdt" ]; then
-        [ "$wdt" = "iTCO_wdt" ] && instmods lpc_ich &&
-            echo "rd.driver.pre=lpc_ich,iTCO_wdt " >> ${initdir}/etc/cmdline.d/00-wdt.conf
-        instmods $wdt
-    fi
-}
