@@ -36,13 +36,15 @@ from urllib import quote, unquote
 import signal
 import crypt
 import random
+import functools
+from distutils import spawn
 
 from pyanaconda.flags import flags
-from pyanaconda.constants import DRACUT_SHUTDOWN_EJECT, TRANSLATIONS_UPDATE_DIR, UNSUPPORTED_HW
+from pyanaconda.constants import DRACUT_SHUTDOWN_EJECT, TRANSLATIONS_UPDATE_DIR, UNSUPPORTED_HW, IPMI_ABORTED
 from pyanaconda.constants import SCREENSHOTS_DIRECTORY, SCREENSHOTS_TARGET_DIRECTORY, SALT_CHARS
 from pyanaconda.regexes import URL_PARSE
-
 from pyanaconda.i18n import _
+from pykickstart.constants import KS_SCRIPT_ONERROR
 
 import logging
 log = logging.getLogger("anaconda")
@@ -179,6 +181,13 @@ def startProgram(argv, root='/', stdin=None, stdout=subprocess.PIPE, stderr=subp
     if env_add:
         env.update(env_add)
 
+    # Try to find absolute path of the executable so we prevent possible
+    # deadlock on strerror called from fork from thread (rhbz#1411407)
+    if not argv[0].startswith('/') and (not target_root or target_root == '/'):
+        exec_path = spawn.find_executable(argv[0])
+        if exec_path != None:
+            argv[0] = exec_path
+
     return subprocess.Popen(argv,
                             stdin=stdin,
                             stdout=stdout,
@@ -1173,6 +1182,19 @@ def ipmi_report(event):
 
     os.remove(path)
 
+def ipmi_abort(scripts=None):
+    ipmi_report(IPMI_ABORTED)
+    runOnErrorScripts(scripts)
+
+def runOnErrorScripts(scripts):
+    if not scripts:
+        return
+
+    log.info("Running kickstart %%onerror script(s)")
+    for script in filter(lambda s: s.type == KS_SCRIPT_ONERROR, scripts):
+        script.run("/")
+    log.info("All kickstart %%onerror script(s) have been run")
+
 def get_platform_groupid():
     """ Return a platform group id string
 
@@ -1242,3 +1264,25 @@ def touch(file_path):
     # even when the path points to dirrectory
     if not os.path.exists(file_path):
         os.mknod(file_path)
+
+def firstNotNone(iterable):
+    """Return the first item that is not None."""
+    return next((item for item in iterable if item is not None), None)
+
+def synchronized(wrapped):
+    """A locking decorator for methods.
+
+    The decorator is only intended for methods and the class providing
+    the method also needs to have a Lock/RLock instantiated in self._lock.
+
+    The decorator prevents the wrapped method from being executed until
+    self._lock can be acquired. Once available, it acquires the lock and
+    prevents other decorated methods & other users of self._lock from
+    executing until the wrapped method finishes running.
+    """
+
+    @functools.wraps(wrapped)
+    def _wrapper(self, *args, **kwargs):
+        with self._lock:
+            return wrapped(self, *args, **kwargs)
+    return _wrapper
