@@ -47,6 +47,9 @@ from pyanaconda.flags import flags, can_touch_runtime_system
 from pyanaconda.i18n import _
 from pyanaconda.regexes import HOSTNAME_PATTERN_WITHOUT_ANCHORS, IBFT_CONFIGURED_DEVICE_NAME
 
+import gi
+gi.require_version("NM", "1.0")
+
 from gi.repository import NetworkManager
 # Register the types so that the use in threads is safe
 NetworkManager.State # pylint: disable=pointless-statement
@@ -269,6 +272,78 @@ class IfcfgFile(SimpleConfigFile):
         ifcfglog.debug("IfcfgFile.unset %s: %s", self.filename, args)
         SimpleConfigFile.unset(self, *args)
 
+def ensure_single_initramfs_connections():
+    """Ensure device configured in initramfs has no more than one NM connection.
+
+    In case of multiple connections for device having ifcfg configuration from
+    boot options, the connection should correspond to the ifcfg file.
+    NetworkManager can be generating additional in-memory connection in case it
+    fails to match device configuration to the ifcfg (#1433891).  By
+    reactivating the device with ifcfg connection the generated in-memory
+    connection will be deleted by NM.
+
+    Don't enforce on slave devices for which having multiple connections can be
+    valid (slave connection, regular device connection).
+    """
+
+    rv = []
+
+    for dev_name in nm.nm_devices():
+        try:
+            nm.nm_device_setting_value(dev_name, "connection", "uuid")
+        except nm.SettingsNotFoundError:
+            pass
+        except nm.MultipleSettingsFoundError:
+            if nm.nm_device_is_slave(dev_name):
+                continue
+
+            ifcfg_path = find_ifcfg_file_of_device(dev_name)
+            if not ifcfg_path:
+                log.error("network: multiple settings but no ifcfg for %s", dev_name)
+                continue
+
+            # Handle only ifcfgs created from boot options in initramfs
+            # (Kickstart based ifcfgs are handled in apply_kickstart)
+            if ifcfg_is_from_kickstart(ifcfg_path):
+                continue
+
+            ensure_active_ifcfg_connection_for_device(ifcfg_path, dev_name, only_replace=True)
+            rv.append(dev_name)
+
+    return rv
+
+def ensure_active_ifcfg_connection_for_device(ifcfg_path, dev_name, only_replace=False):
+    """Make sure active connection of a device is the one of ifcfg file
+
+    :param ifcfg_path: path of ifcfg file with the connection to be used
+    :type ifcfg_path: str
+    :param dev_name: name of device to apply the connection to
+    :type dev_name: str
+    :param only_replace: apply the connection only if the device has different
+                         active connection
+    :type only_replace: bool
+
+    """
+    msg = "not activating"
+    ifcfg = IfcfgFile(ifcfg_path)
+    ifcfg.read()
+    con_uuid = ifcfg.get("UUID")
+    active_con_uuid = nm.nm_device_active_con_uuid(dev_name)
+    if active_con_uuid or not only_replace:
+        if con_uuid != active_con_uuid:
+            msg = "activating"
+            try:
+                nm.nm_activate_device_connection(dev_name, con_uuid)
+            except nm.UnknownConnectionError as e:
+                log.warning("network: can't activate connection %s on %s: %s",
+                            con_uuid, dev_name, e)
+    log.debug("network: ensure active ifcfg connection for %s (%s -> %s): %s",
+               dev_name, active_con_uuid, con_uuid, msg)
+
+def ifcfg_is_from_kickstart(path):
+    with open(path, 'r') as f:
+        return "Generated by parse-kickstart" in f.read()
+
 def dumpMissingDefaultIfcfgs():
     """
     Dump missing default ifcfg file for wired devices.
@@ -277,20 +352,23 @@ def dumpMissingDefaultIfcfgs():
     rv = []
 
     for devname in nm.nm_devices():
-        if not nm.nm_device_type_is_ethernet(devname):
+        if not nm.nm_device_type_is_ethernet(devname) \
+           and not nm.nm_device_type_is_infiniband(devname):
+            continue
+        if find_ifcfg_file_of_device(devname):
             continue
-
         try:
             nm.nm_device_setting_value(devname, "connection", "uuid")
         except nm.SettingsNotFoundError:
-            if find_ifcfg_file_of_device(devname):
-                continue
             from pyanaconda.kickstart import AnacondaKSHandler
             handler = AnacondaKSHandler()
             # pylint: disable=E1101
             network_data = handler.NetworkData(onboot=False, ipv6="auto")
             add_connection_for_ksdata(network_data, devname)
             rv.append(devname)
+        except nm.MultipleSettingsFoundError as e:
+            if not nm.nm_device_is_slave(devname):
+                log.debug("network: %s while checking missing ifcfgs, device %s", e, devname)
 
     return rv
 
@@ -374,6 +452,33 @@ def dracutBootArguments(devname, ifcfg, storage_ipaddr, hostname=None):
             netargs.add("team=%s:%s" % (devname,
                                         ",".join(dev for dev, _cfg in slaves)))
 
+        if ifcfg.get("TYPE") == "Vlan":
+            physdev_spec = ifcfg.get("PHYSDEV")
+            physdev = None
+            if physdev_spec in nm.nm_devices():
+                physdev = physdev_spec
+                ifcfg_path = find_ifcfg_file_of_device(physdev)
+                if ifcfg_path:
+                    ifcfg = IfcfgFile(ifcfg_path)
+                    ifcfg.read()
+                else:
+                    log.debug("network: can't find ifcfg of vlan parent %s", physdev)
+            # physical device can be specified by connection uuid (eg from nm-c-e)
+            else:
+                ifcfg_path = find_ifcfg_file([("UUID", physdev_spec)])
+                if ifcfg_path:
+                    ifcfg = IfcfgFile(ifcfg_path)
+                    ifcfg.read()
+                    # On s390 with net.ifnames=0 there is no DEVICE
+                    physdev = ifcfg.get("DEVICE") or ifcfg.get("NAME")
+
+            if physdev:
+                netargs.add("vlan=%s:%s" % (devname, physdev))
+            else:
+                log.warning("network: can't find parent of vlan device %s specified by %s",
+                             devname, physdev_spec)
+
+    # For vlan ifcfg now refers to the physical device file
     nettype = ifcfg.get("NETTYPE")
     subchannels = ifcfg.get("SUBCHANNELS")
     if blivet.arch.isS390() and nettype and subchannels:
@@ -392,12 +497,13 @@ def _get_ip_setting_values_from_ksdata(networkdata):
     # ipv4 settings
     if networkdata.noipv4:
         method4 = "disabled"
+    elif networkdata.bootProto == "static":
+        method4 = "manual"
     else:
         method4 = "auto"
-        if networkdata.bootProto == "static":
-            method4 = "manual"
-        values.append(["ipv4", "method", method4, "s"])
+    values.append(["ipv4", "method", method4, "s"])
 
+    addresses4 = []
     if method4 == "manual":
         addr4 = nm.nm_ipv4_to_dbus_int(networkdata.ip)
         if networkdata.gateway:
@@ -405,7 +511,9 @@ def _get_ip_setting_values_from_ksdata(networkdata):
         else:
             gateway4 = 0 # will be ignored by NetworkManager
         prefix4 = netmask2prefix(networkdata.netmask)
-        values.append(["ipv4", "addresses", [[addr4, prefix4, gateway4]], "aau"])
+        addresses4 = [[addr4, prefix4, gateway4]]
+
+    values.append(["ipv4", "addresses", addresses4, "aau"])
 
     # ipv6 settings
     if networkdata.noipv6:
@@ -421,6 +529,7 @@ def _get_ip_setting_values_from_ksdata(networkdata):
             method6 = "manual"
     values.append(["ipv6", "method", method6, "s"])
 
+    addresses6 = []
     if method6 == "manual":
         addr6, _slash, prefix6 = networkdata.ipv6.partition("/")
         if prefix6:
@@ -432,7 +541,8 @@ def _get_ip_setting_values_from_ksdata(networkdata):
             gateway6 = nm.nm_ipv6_to_dbus_ay(networkdata.ipv6gateway)
         else:
             gateway6 = [0] * 16
-        values.append(["ipv6", "addresses", [(addr6, prefix6, gateway6)], "a(ayuay)"])
+        addresses6 = [(addr6, prefix6, gateway6)]
+    values.append(["ipv6", "addresses", addresses6, "a(ayuay)"])
 
     # nameservers
     nss4 = []
@@ -451,10 +561,14 @@ def _get_ip_setting_values_from_ksdata(networkdata):
     return values
 
 def update_settings_with_ksdata(devname, networkdata):
+    try:
+        uuid = nm.nm_device_setting_value(devname, "connection", "uuid")
+    except nm.MultipleSettingsFoundError as e:
+        uuid = find_ifcfg_uuid_of_device(devname)
+        log.debug("network: %s for %s, using %s", e, devname, uuid)
     new_values = _get_ip_setting_values_from_ksdata(networkdata)
     new_values.append(['connection', 'autoconnect', False, 'b'])
-    uuid = nm.nm_device_setting_value(devname, "connection", "uuid")
-    nm.nm_update_settings_of_device(devname, new_values)
+    nm.nm_update_settings(uuid, new_values)
     return uuid
 
 def bond_options_ksdata_to_dbus(opts_str):
@@ -470,6 +584,7 @@ def add_connection_for_ksdata(networkdata, devname):
     con_uuid = str(uuid4())
     values = _get_ip_setting_values_from_ksdata(networkdata)
     # HACK preventing NM to autoactivate the connection
+    # The real network --onboot value (ifcfg ONBOOT) will be set later by setOnboot
     #values.append(['connection', 'autoconnect', networkdata.onboot, 'b'])
     values.append(['connection', 'autoconnect', False, 'b'])
     values.append(['connection', 'uuid', con_uuid, 's'])
@@ -482,8 +597,8 @@ def add_connection_for_ksdata(networkdata, devname):
         values.append(['bond', 'interface-name', devname, 's'])
         options = bond_options_ksdata_to_dbus(networkdata.bondopts)
         values.append(['bond', 'options', options, 'a{ss}'])
-        for slave in networkdata.bondslaves.split(","):
-            suuid = _add_slave_connection('bond', slave, devname, networkdata.activate)
+        for i, slave in enumerate(networkdata.bondslaves.split(","), 1):
+            suuid = _add_slave_connection('bond', i, slave, devname, networkdata.activate)
             added_connections.append((suuid, slave))
         dev_spec = None
     # type "team"
@@ -492,11 +607,11 @@ def add_connection_for_ksdata(networkdata, devname):
         values.append(['connection', 'id', devname, 's'])
         values.append(['team', 'interface-name', devname, 's'])
         values.append(['team', 'config', networkdata.teamconfig, 's'])
-        for (slave, cfg) in networkdata.teamslaves:
+        for i, (slave, cfg) in enumerate(networkdata.teamslaves, 1):
             svalues = []
             if cfg:
                 svalues.append(['team-port', 'config', cfg, 's'])
-            suuid = _add_slave_connection('team', slave, devname, networkdata.activate, svalues)
+            suuid = _add_slave_connection('team', i, slave, devname, networkdata.activate, svalues)
             added_connections.append((suuid, slave))
         dev_spec = None
     # type "vlan"
@@ -527,8 +642,8 @@ def add_connection_for_ksdata(networkdata, devname):
                 log.error("Invalid bridge option %s", opt)
                 continue
             values.append(['bridge', key, int(value), 'u'])
-        for slave in networkdata.bridgeslaves.split(","):
-            suuid = _add_slave_connection('bridge', slave, devname, networkdata.activate)
+        for i, slave in enumerate(networkdata.bridgeslaves.split(","), 1):
+            suuid = _add_slave_connection('bridge', i, slave, devname, networkdata.activate)
             added_connections.append((suuid, slave))
         dev_spec = None
     # type "infiniband"
@@ -636,10 +751,9 @@ def _get_s390_settings(devname):
 
     return cfg
 
-def _add_slave_connection(slave_type, slave, master, activate, values=None):
+def _add_slave_connection(slave_type, slave_idx, slave, master, activate, values=None):
     values = values or []
-    #slave_name = "%s slave %d" % (devname, slave_idx)
-    slave_name = slave
+    slave_name = "%s slave %d" % (master, slave_idx)
 
     suuid =  str(uuid4())
     # assume ethernet, TODO: infiniband, wifi, vlan
@@ -648,9 +762,10 @@ def _add_slave_connection(slave_type, slave, master, activate, values=None):
     values.append(['connection', 'slave-type', slave_type, 's'])
     values.append(['connection', 'master', master, 's'])
     values.append(['connection', 'type', '802-3-ethernet', 's'])
-    mac = nm.nm_device_perm_hwaddress(slave)
-    mac = [int(b, 16) for b in mac.split(":")]
-    values.append(['802-3-ethernet', 'mac-address', mac, 'ay'])
+    values.append(['connection', 'interface-name', slave, 's'])
+    # HACK preventing NM to autoactivate the connection
+    # The real network --onboot value (ifcfg ONBOOT) will be set later by setOnboot
+    values.append(['connection', 'autoconnect', False, 'b'])
 
     # disconnect slaves
     if activate:
@@ -658,10 +773,6 @@ def _add_slave_connection(slave_type, slave, master, activate, values=None):
             nm.nm_disconnect_device(slave)
         except nm.DeviceNotActiveError:
             pass
-    # remove ifcfg file
-    ifcfg_path = find_ifcfg_file_of_device(slave)
-    if ifcfg_path and os.access(ifcfg_path, os.R_OK):
-        os.unlink(ifcfg_path)
 
     nm.nm_add_connection(values)
 
@@ -697,6 +808,8 @@ def ksdata_from_ifcfg(devname, uuid=None):
     if devname in nm.nm_devices():
         if nm.nm_device_type_is_ethernet(devname):
             nd.device = devname
+        elif nm.nm_device_type_is_infiniband(devname):
+            nd.device = devname
         elif nm.nm_device_type_is_wifi(devname):
             nm.device = ""
         elif nm.nm_device_type_is_bond(devname):
@@ -847,7 +960,11 @@ def ifcfg_to_ksdata(ifcfg, devname):
         for dev, cfg in slaves:
             nd.teamslaves.append((dev, cfg))
 
-        teamconfig = nm.nm_device_setting_value(devname, "team", "config")
+        try:
+            teamconfig = nm.nm_device_setting_value(devname, "team", "config")
+        except nm.MultipleSettingsFoundError as e:
+            teamconfig = None
+            log.debug("network: %s while looking for team device config", e)
         if teamconfig:
             nd.teamconfig = teamconfig
 
@@ -915,6 +1032,17 @@ def find_ifcfg_file_of_device(devname, root_path=""):
 
     return ifcfg_path
 
+def find_ifcfg_uuid_of_device(devname):
+    ifcfg_path = find_ifcfg_file_of_device(devname)
+    if ifcfg_path:
+        ifcfg = IfcfgFile(ifcfg_path)
+        ifcfg.read()
+        uuid = ifcfg.get('UUID')
+    else:
+        log.debug("network: can't find ifcfg file of %s", devname)
+        uuid = None
+    return uuid
+
 def find_ifcfg_file(values, root_path=""):
     for filepath in _ifcfg_files(os.path.normpath(root_path+netscriptsDir)):
         ifcfg = IfcfgFile(filepath)
@@ -995,17 +1123,33 @@ def get_team_slaves(master_specs):
 
 def ibftIface():
     iface = ""
-    ipopt = flags.cmdline.get('ip')
+    ipopts = flags.cmdline.get('ip')
     # Examples (dhcp, static):
     # ibft0:dhcp
     # 10.34.102.244::10.34.102.54:255.255.255.0::ibft0:none
-    if ipopt:
-        for item in ipopt.split(":"):
-            if item.startswith('ibft'):
-                iface = item
-                break
+    if ipopts:
+        for ipopt in ipopts.split(" "):
+            for item in ipopt.split(":"):
+                if item.startswith('ibft'):
+                    iface = item
+                    break
     return iface
 
+def hostname_from_cmdline(cmdline):
+    # legacy hostname= option
+    hostname = flags.cmdline.get('hostname', "")
+    # ip= option
+    ipopts = flags.cmdline.get('ip')
+    # Example (2 options):
+    # ens3:dhcp 10.34.102.244::10.34.102.54:255.255.255.0:myhostname:ens9:none
+    if ipopts:
+        for ipopt in ipopts.split(" "):
+            try:
+                hostname = ipopt.split(':')[4]
+            except IndexError:
+                pass
+    return hostname
+
 def ifaceForHostIP(host):
     route = iutil.execWithCapture("ip", [ "route", "get", "to", host ])
     if not route:
@@ -1125,14 +1269,17 @@ def write_hostname(rootpath, ksdata, overwrite=False):
 
 def disableIPV6(rootpath):
     cfgfile = os.path.normpath(rootpath + ipv6ConfFile)
-    if ('noipv6' in flags.cmdline
-        and all(nm.nm_device_setting_value(dev, "ipv6", "method") == "ignore"
-                for dev in nm.nm_devices() if nm.nm_device_type_is_ethernet(dev))):
-        log.info('Disabling ipv6 on target system')
-        with open(cfgfile, "a") as f:
-            f.write("# Anaconda disabling ipv6 (noipv6 option)\n")
-            f.write("net.ipv6.conf.all.disable_ipv6=1\n")
-            f.write("net.ipv6.conf.default.disable_ipv6=1\n")
+    try:
+        if ('noipv6' in flags.cmdline
+            and all(nm.nm_device_setting_value(dev, "ipv6", "method") == "ignore"
+                    for dev in nm.nm_devices() if nm.nm_device_type_is_ethernet(dev))):
+            log.info('Disabling ipv6 on target system')
+            with open(cfgfile, "a") as f:
+                f.write("# Anaconda disabling ipv6 (noipv6 option)\n")
+                f.write("net.ipv6.conf.all.disable_ipv6=1\n")
+                f.write("net.ipv6.conf.default.disable_ipv6=1\n")
+    except nm.MultipleSettingsFoundError as e:
+        log.debug("network: %s in disableIPV6", e)
 
 # sets ONBOOT=yes (and its mirror value in ksdata) for devices used by FCoE
 def autostartFCoEDevices(rootpath, storage, ksdata):
@@ -1243,10 +1390,14 @@ def setOnboot(ksdata):
                 except (nm.SettingsNotFoundError, nm.UnknownDeviceError) as e:
                     log.debug("setOnboot: %s", e)
                     continue
+                except nm.MultipleSettingsFoundError:
+                    # In case of multiple connections for a device, update ifcfg directly
+                    if not update_onboot_value(devname, network_data.onboot, root_path=""):
+                        continue
 
             updated_devices.append(devname)
 
-        if network_data.bondslaves or network_data.teamslaves:
+        if network_data.bondslaves or network_data.teamslaves or network_data.bridgeslaves:
             updated_slaves = update_slaves_onboot_value(master, network_data.onboot)
             updated_devices.extend(updated_slaves)
 
@@ -1261,6 +1412,9 @@ def apply_kickstart(ksdata):
         if network_data.essid:
             continue
 
+        if network_data.activate is None and i == 0:
+            network_data.activate = True
+
         dev_name = get_device_name(network_data)
         if not dev_name:
             log.warning("network: apply kickstart: --device %s does not exist", network_data.device)
@@ -1268,25 +1422,10 @@ def apply_kickstart(ksdata):
 
         ifcfg_path = find_ifcfg_file_of_device(dev_name)
         if ifcfg_path:
-            with open(ifcfg_path, 'r') as f:
-                # If we have kickstart ifcfg from initramfs
-                if "Generated by parse-kickstart" in f.read():
-                    # and we should activate the device
-                    if network_data.activate or (i == 0 and network_data.activate is None):
-                        ifcfg = IfcfgFile(ifcfg_path)
-                        ifcfg.read()
-                        con_uuid = ifcfg.get("UUID")
-                        # and the ifcfg had not been already applied to device by NM
-                        if con_uuid != nm.nm_device_active_con_uuid(dev_name):
-                            # apply it overriding configuration generated by NM
-                            # taking over connection activated in initramfs
-                            log.debug("network: kickstart - reactivating device %s with %s", dev_name, con_uuid)
-                            try:
-                                nm.nm_activate_device_connection(dev_name, con_uuid)
-                            except nm.UnknownConnectionError:
-                                log.warning("network: kickstart - can't activate connection %s on %s",
-                                            con_uuid, dev_name)
-                    continue
+            if ifcfg_is_from_kickstart(ifcfg_path):
+                if network_data.activate:
+                    ensure_active_ifcfg_connection_for_device(ifcfg_path, dev_name)
+                continue
 
         # If we don't have kickstart ifcfg from initramfs the command was added
         # in %pre section after switch root, so apply it now
@@ -1304,6 +1443,7 @@ def apply_kickstart(ksdata):
         if network_data.activate:
             for con_uuid, dev_name in added_connections:
                 try:
+                    log.debug("network: pre kickstart - activating connection %s for %s", con_uuid, dev_name)
                     nm.nm_activate_device_connection(dev_name, con_uuid)
                 except (nm.UnknownConnectionError, nm.UnknownDeviceError) as e:
                     log.warning("network: pre kickstart: can't activate connection %s on %s: %s",
@@ -1317,11 +1457,19 @@ def networkInitialize(ksdata):
     log.debug("network: devices found %s", nm.nm_devices())
     logIfcfgFiles("network initialization")
 
+    log.debug("network: ensure single initramfs connections")
+    devnames = ensure_single_initramfs_connections()
+    if devnames:
+        msg = "single connection ensured for devices %s" % devnames
+        log.debug("network: %s", msg)
+        logIfcfgFiles(msg)
+    log.debug("network: apply kickstart")
     devnames = apply_kickstart(ksdata)
     if devnames:
         msg = "kickstart pre section applied for devices %s" % devnames
         log.debug("network: %s", msg)
         logIfcfgFiles(msg)
+    log.debug("network: create missing ifcfg files")
     devnames = dumpMissingDefaultIfcfgs()
     if devnames:
         msg = "missing ifcfgs created for devices %s" % devnames
@@ -1331,14 +1479,17 @@ def networkInitialize(ksdata):
     # For kickstart network --activate option we set ONBOOT=yes
     # in dracut to get devices activated by NM. The real network --onboot
     # value is set here.
+    log.debug("network: set real ONBOOT value")
     devnames = setOnboot(ksdata)
     if devnames:
         msg = "real kickstart ONBOOT value set for devices %s" % devnames
         log.debug("network: %s", msg)
         logIfcfgFiles(msg)
 
+    # initialize ksdata hostname
     if ksdata.network.hostname is None:
-        update_hostname_data(ksdata, DEFAULT_HOSTNAME)
+        hostname = hostname_from_cmdline(flags.cmdline) or DEFAULT_HOSTNAME
+        update_hostname_data(ksdata, hostname)
 
 def _get_ntp_servers_from_dhcp(ksdata):
     """Check if some NTP servers were returned from DHCP and set them
@@ -1361,33 +1512,50 @@ def _get_ntp_servers_from_dhcp(ksdata):
         # no NTP servers were specified, add those from DHCP
         ksdata.timezone.ntpservers = hostnames
 
-def _wait_for_connecting_NM():
-    """If NM is in connecting state, wait for connection.
-    Return value: NM has got connection."""
+def wait_for_connected_NM(timeout=constants.NETWORK_CONNECTION_TIMEOUT, only_connecting=False):
+    """Wait for NM being connected.
+
+    If only_connecting is set, wait only if NM is in connecting state and
+    return immediately after leaving this state (regardless of the new state).
+    Used to wait for dhcp configuration in progress.
+
+    :param timeout: timeout in seconds
+    :type timeout: int
+    :parm only_connecting: wait only for the result of NM being connecting
+    :type only_connecting: bool
+    :return: NM is connected
+    :rtype: bool
+    """
 
     if nm.nm_is_connected():
         return True
 
-    if nm.nm_is_connecting():
-        log.debug("waiting for connecting NM (dhcp?)")
+    if only_connecting:
+        if nm.nm_is_connecting():
+            log.debug("network: waiting for connecting NM (dhcp in progress?), timeout=%d", timeout)
+        else:
+            return False
     else:
-        return False
+        log.debug("network: waiting for connected NM, timeout=%d", timeout)
 
     i = 0
-    while nm.nm_is_connecting() and i < constants.NETWORK_CONNECTION_TIMEOUT:
+    while i < timeout:
         i += constants.NETWORK_CONNECTED_CHECK_INTERVAL
         time.sleep(constants.NETWORK_CONNECTED_CHECK_INTERVAL)
         if nm.nm_is_connected():
-            log.debug("connected, waited %d seconds", i)
+            log.debug("network: NM connected, waited %d seconds", i)
             return True
+        elif only_connecting:
+            if not nm.nm_is_connecting():
+                break
 
-    log.debug("not connected, waited %d of %d secs", i, constants.NETWORK_CONNECTION_TIMEOUT)
+    log.debug("network: NM not connected, waited %d seconds", i)
     return False
 
 def wait_for_network_devices(devices, timeout=constants.NETWORK_CONNECTION_TIMEOUT):
     devices = set(devices)
     i = 0
-    log.debug("waiting for connection of devices %s for iscsi", devices)
+    log.debug("network: waiting for connection of devices %s for iscsi", devices)
     while  i < timeout:
         if not devices - set(nm.nm_activated_devices()):
             return True
@@ -1396,10 +1564,14 @@ def wait_for_network_devices(devices, timeout=constants.NETWORK_CONNECTION_TIMEO
     return False
 
 def wait_for_connecting_NM_thread(ksdata):
-    """This function is called from a thread which is run at startup
-    to wait for Network Manager to connect."""
-    # connection (e.g. auto default dhcp) is activated by NM service
-    connected = _wait_for_connecting_NM()
+    """Wait for connecting NM in thread, do some work and signal connectivity
+
+    This function is called from a thread which is run at startup to wait for
+    NetworkManager being in connecting state (eg getting IP from DHCP). When NM
+    leaves connecting state do some actions and signal new state if NM becomes
+    connected.
+    """
+    connected = wait_for_connected_NM(only_connecting=True)
     if connected:
         _get_ntp_servers_from_dhcp(ksdata)
     with network_connected_condition:
@@ -1455,7 +1627,8 @@ def status_message():
 
             if len(nonslaves) == 1:
                 devname = nonslaves[0]
-                if nm.nm_device_type_is_ethernet(devname):
+                if nm.nm_device_type_is_ethernet(devname) \
+                   or nm.nm_device_type_is_infiniband(devname):
                     msg = _("Wired (%(interface_name)s) connected") \
                           % {"interface_name": devname}
                 elif nm.nm_device_type_is_wifi(devname):
@@ -1474,14 +1647,19 @@ def status_message():
                           % {"interface_name": devname, \
                              "list_of_slaves": ",".join(slaves[devname])}
                 elif nm.nm_device_type_is_vlan(devname):
-                    parent = nm.nm_device_setting_value(devname, "vlan", "parent")
-                    vlanid = nm.nm_device_setting_value(devname, "vlan", "id")
+                    try:
+                        parent = nm.nm_device_setting_value(devname, "vlan", "parent")
+                        vlanid = nm.nm_device_setting_value(devname, "vlan", "id")
+                    except nm.MultipleSettingsFoundError as e:
+                        parent = vlanid = None
+                        log.debug("network: %s when looking for vlan settings of %s", e, devname)
                     msg = _("VLAN %(interface_name)s (%(parent_device)s, ID %(vlanid)s) connected") \
                           % {"interface_name": devname, "parent_device": parent, "vlanid": vlanid}
             elif len(nonslaves) > 1:
                 devlist = []
                 for devname in nonslaves:
-                    if nm.nm_device_type_is_ethernet(devname):
+                    if nm.nm_device_type_is_ethernet(devname) \
+                       or nm.nm_device_type_is_infiniband(devname):
                         devlist.append("%s" % devname)
                     elif nm.nm_device_type_is_wifi(devname):
                         devlist.append("%s" % ssids[devname])
@@ -1532,19 +1710,19 @@ def update_slaves_onboot_value(devname, value):
         uuid = nm.nm_device_setting_value(devname, "connection", "uuid")
     except nm.UnknownDeviceError:
         # Until activated, the device does not exist, so look in its ifcfg file
-        ifcfg_path = find_ifcfg_file_of_device(devname)
-        if not ifcfg_path:
-            log.debug("network: can't find ifcfg file of %s", devname)
+        uuid = find_ifcfg_uuid_of_device(devname)
+        if not uuid:
             return retval
-        ifcfg = IfcfgFile(ifcfg_path)
-        ifcfg.read()
-        uuid = ifcfg.get('UUID')
+
+    except nm.MultipleSettingsFoundError as e:
+        uuid = None
+        log.debug("network: %s when updating onboot value of slave %s", e, devname)
 
     # Find and update ifcfg files of slaves
     for filepath in _ifcfg_files(netscriptsDir):
         ifcfg = IfcfgFile(filepath)
         ifcfg.read()
-        master = ifcfg.get("MASTER") or ifcfg.get("TEAM_MASTER")
+        master = ifcfg.get("MASTER") or ifcfg.get("TEAM_MASTER") or ifcfg.get("BRIDGE")
         if master in (devname, uuid):
             ifcfg.set(('ONBOOT', ifcfg_value))
             ifcfg.write()
