@@ -19,8 +19,13 @@
 # Red Hat Author(s): Radek Vykydal <rvykydal@redhat.com>
 #
 
-from gi.repository import Gio, GLib
+import gi
+gi.require_version("GLib", "2.0")
+gi.require_version("Gio", "2.0")
+gi.require_version("NM", "1.0")
+gi.require_version("NetworkManager", "1.0")
 
+from gi.repository import Gio, GLib
 from gi.repository import NetworkManager
 # Register the types so that the use in threads is safe
 NetworkManager.ActiveConnectionState # pylint: disable=pointless-statement
@@ -73,6 +78,11 @@ class SettingsNotFoundError(ValueError):
     def __str__(self):
         return self.__repr__()
 
+class MultipleSettingsFoundError(ValueError):
+    """Too many NMRemoteConnection objects were found"""
+    def __str__(self):
+        return self.__repr__()
+
 class UnknownMethodGetError(Exception):
     """Object does not have Get, most probably being invalid"""
     def __str__(self):
@@ -126,9 +136,11 @@ def _get_property(object_path, prop, interface_name_suffix=""):
     try:
         prop = proxy.Get('(ss)', interface_name, prop)
     except GLib.GError as e:
+        # pylint: disable=unsupported-membership-test
         if ("org.freedesktop.DBus.Error.AccessDenied" in e.message or
             "org.freedesktop.DBus.Error.InvalidArgs" in e.message):
             return None
+        # pylint: disable=unsupported-membership-test
         elif "org.freedesktop.DBus.Error.UnknownMethod" in e.message:
             raise UnknownMethodGetError
         else:
@@ -265,6 +277,7 @@ def nm_device_property(name, prop):
     try:
         device = proxy.GetDeviceByIpIface('(s)', name)
     except GLib.GError as e:
+        # pylint: disable=unsupported-membership-test
         if "org.freedesktop.NetworkManager.UnknownDevice" in e.message:
             raise UnknownDeviceError(name, e)
         raise
@@ -738,7 +751,11 @@ def _find_settings(value, key1, key2, format_value=lambda x:x):
     connections = proxy.ListConnections()
     for con in connections:
         proxy = _get_proxy(object_path=con, interface_name="org.freedesktop.NetworkManager.Settings.Connection")
-        settings = proxy.GetSettings()
+        try:
+            settings = proxy.GetSettings()
+        except GLib.GError as e:
+            log.debug("Exception raised in _find_settings: %s", e)
+            continue
         try:
             v = settings[key1][key2]
         except KeyError:
@@ -771,7 +788,11 @@ def nm_get_all_settings():
     connections = proxy.ListConnections()
     for con in connections:
         proxy = _get_proxy(object_path=con, interface_name="org.freedesktop.NetworkManager.Settings.Connection")
-        settings = proxy.GetSettings()
+        try:
+            settings = proxy.GetSettings()
+        except GLib.GError as e:
+            log.debug("Exception raised in nm_get_all_settings: %s", e)
+            continue
         retval.append(settings)
 
     return retval
@@ -792,14 +813,21 @@ def nm_device_setting_value(name, key1, key2):
        :raise UnknownDeviceError: if device is not found
        :raise SettingsNotFoundError: if settings were not found
                                            (eg for "wlan0")
+       :raise MultipleSettingsFoundError: if multiple settings were found
     """
     settings_paths = _device_settings(name)
     if not settings_paths:
         raise SettingsNotFoundError(name)
+    elif len(settings_paths) > 1:
+        raise MultipleSettingsFoundError(name)
     else:
         settings_path = settings_paths[0]
     proxy = _get_proxy(object_path=settings_path, interface_name="org.freedesktop.NetworkManager.Settings.Connection")
-    settings = proxy.GetSettings()
+    try:
+        settings = proxy.GetSettings()
+    except Exception as e:
+        log.debug("nm_device_setting_value: %s", e)
+        raise SettingsNotFoundError(name)
     try:
         value = settings[key1][key2]
     except KeyError:
@@ -844,6 +872,7 @@ def nm_disconnect_device(name):
     try:
         device = proxy.GetDeviceByIpIface('(s)', name)
     except GLib.GError as e:
+        # pylint: disable=unsupported-membership-test
         if "org.freedesktop.NetworkManager.UnknownDevice" in e.message:
             raise UnknownDeviceError(name, e)
         raise
@@ -852,6 +881,7 @@ def nm_disconnect_device(name):
     try:
         device_proxy.Disconnect()
     except GLib.GError as e:
+        # pylint: disable=unsupported-membership-test
         if "org.freedesktop.NetworkManager.Device.NotActive" in e.message:
             raise DeviceNotActiveError(name, e)
         raise
@@ -926,6 +956,7 @@ def nm_add_connection(values):
     try:
         connection = proxy.AddConnection('(a{sa{sv}})', settings)
     except GLib.GError as e:
+        # pylint: disable=unsupported-membership-test
         if "bond.options: invalid option" in e.message:
             raise BondOptionsError(e)
         raise
@@ -946,6 +977,37 @@ def nm_delete_connection(uuid):
     proxy = _get_proxy(object_path=settings_paths[0], interface_name="org.freedesktop.NetworkManager.Settings.Connection")
     proxy.Delete()
 
+
+def nm_update_settings(uuid, new_values):
+    """Update settings of connection given by uuid.
+
+       The type of value is determined from existing settings of device.
+       If setting for key1, key2 does not exist, default_type_str is used or
+       if None, the type is inferred from the value supplied (string and bool only).
+
+       :param uuid: uuid of the connection
+       :type name: str
+       :param new_values: list of settings with new values and its types
+                          [[key1, key2, value, default_type_str]]
+                          key1: first-level key of setting (eg "connection")
+                          key2: second-level key of setting (eg "uuid")
+                          value: new value
+                          default_type_str: dbus type of new value to be used
+                                            if the setting does not already exist;
+                                            if None, the type is inferred from
+                                            value (string and bool only)
+       :type new_values: [[key1, key2, value, default_type_str], ...]
+                         key1: str
+                         key2: str
+                         value:
+                         default_type_str: str
+       :raise SettingsNotFoundError: if settings were not found (eg for "wlan0")
+    """
+    settings_paths = _find_settings(uuid, "connection", "uuid")
+    if not settings_paths:
+        raise SettingsNotFoundError(uuid)
+    return _update_settings(settings_paths[0], new_values)
+
 def nm_update_settings_of_device(name, new_values):
     """Update setting of device.
 
@@ -972,10 +1034,13 @@ def nm_update_settings_of_device(name, new_values):
        :raise UnknownDeviceError: if device is not found
        :raise SettingsNotFoundError: if settings were not found
                                            (eg for "wlan0")
+       :raise MultipleSettingsFoundError: if multiple settings were found
     """
     settings_paths = _device_settings(name)
     if not settings_paths:
         raise SettingsNotFoundError(name)
+    elif len(settings_paths) > 1:
+        raise MultipleSettingsFoundError(name)
     else:
         settings_path = settings_paths[0]
     return _update_settings(settings_path, new_values)
