@@ -32,7 +32,6 @@
                   --proxyPassword
                     - drop the use of a file for proxy and ftp auth info
                 - specified via KS as a URL
-
 """
 
 import ConfigParser
@@ -40,10 +39,12 @@ import os
 import shutil
 import sys
 import time
-from pyanaconda.iutil import execReadlines
+import hashlib
+from pyanaconda.iutil import execReadlines, ipmi_abort
+from pyanaconda.product import productName
 from pyanaconda.simpleconfig import simple_replace
 from functools import wraps
-from pyanaconda.product import productName 
+from urlgrabber.grabber import URLGrabber, URLGrabError
 
 import logging
 log = logging.getLogger("packaging")
@@ -66,7 +67,7 @@ except ImportError:
     yum = None
 
 from pyanaconda.constants import BASE_REPO_NAME, DRACUT_ISODIR, INSTALL_TREE, ISO_DIR, MOUNT_DIR, \
-                                 LOGLVL_LOCK, IPMI_ABORTED
+                                 LOGLVL_LOCK
 from pyanaconda.flags import flags
 
 from pyanaconda import iutil
@@ -119,8 +120,7 @@ class YumLock(object):
             log.log(LOGLVL_LOCK, "gave up _yum_lock for %s", threading.currentThread().name)
 
 def refresh_base_repo(cond_fn=None):
-    """
-    Function returning decorator for methods that invalidate base repo.
+    """Function returning decorator for methods that invalidate base repo.
     After the method has been run the base repo will be refreshed
 
     :param cond_fn: condition function telling if base repo should be
@@ -130,11 +130,9 @@ def refresh_base_repo(cond_fn=None):
     While the method runs the base_repo is set to None.
     """
     def decorator(method):
-        """
-        Decorator for methods that invalidate base repo cache.
+        """Decorator for methods that invalidate base repo cache.
 
         :param method: decorated method of the YumPayload class
-
         """
         @wraps(method)
         def inner_method(yum_payload, *args, **kwargs):
@@ -158,15 +156,13 @@ _yum_installer_langpack_conf = "/tmp/yum.pluginconf.d/langpacks.conf"
 _yum_target_langpack_conf = "/etc/yum/pluginconf.d/langpacks.conf"
 
 class YumPayload(PackagePayload):
-    """ A YumPayload installs packages onto the target system using yum.
+    """A YumPayload installs packages onto the target system using yum.
 
-        User-defined (aka: addon) repos exist both in ksdata and in yum. They
-        are the only repos in ksdata.repo. The repos we find in the yum config
-        only exist in yum. Lastly, the base repo exists in yum and in
-        ksdata.method.
+    User-defined (aka: addon) repos exist both in ksdata and in yum. They
+    are the only repos in ksdata.repo. The repos we find in the yum config
+    only exist in yum. Lastly, the base repo exists in yum and in
+    ksdata.method.
     """
-
-
     def __init__(self, data):
         if rpm is None or yum is None:
             raise PayloadError("unsupported payload type")
@@ -182,10 +178,13 @@ class YumPayload(PackagePayload):
         self._base_repo = None
         self._base_repo_lock = threading.RLock()
 
+        # save repomd metadata
+        self._repoMD_list = []
+
         self.reset()
 
     def reset(self, root=None, releasever=None):
-        """ Reset this instance to its initial (unconfigured) state. """
+        """Reset this instance to its initial (unconfigured) state."""
 
         super(YumPayload, self).reset()
         # This value comes from a default install of the x86_64 Fedora 18.  It
@@ -204,20 +203,37 @@ class YumPayload(PackagePayload):
         self._writeYumConfig()
         self._setup = True
 
+    def postSetup(self):
+        """Save repomd.xml files for later test for availability."""
+        self._repoMD_list = []
+        for repoID in self.repos:
+            if not self.isRepoEnabled(repoID):
+                log.info("Not checking metadata for disabled repo %s", repoID)
+                continue
+
+            repo = self.getRepo(repoID)
+            with _yum_lock:
+                repoMD = RepoMDMetaHash(self, repo)
+                repoMD.storeRepoMDHash()
+                self._repoMD_list.append(repoMD)
+
     def unsetup(self):
         super(YumPayload, self).unsetup()
         self._setup = False
+        self._repoMD_list = []
 
     def _resetYum(self, root=None, keep_cache=False, releasever=None, cache_dir=_yum_cache_dir):
-        """ Delete and recreate the payload's YumBase instance.
+        """Delete and recreate the payload's YumBase instance.
 
-            Setup _yum.preconf -- DO NOT TOUCH IT OUTSIDE THIS METHOD
-            NOTE:  This is enforced by tests/pylint/preconf.py.  If the name
-            of this method changes, change it there too.
+        Setup _yum.preconf -- DO NOT TOUCH IT OUTSIDE THIS METHOD
+        NOTE:  This is enforced by tests/pylint/preconf.py.  If the name
+        of this method changes, change it there too.
         """
         if root is None:
             root = self._root_dir
 
+        self._repoMD_list = []
+
         with _yum_lock:
             if self._yum:
                 if not keep_cache:
@@ -269,7 +285,7 @@ class YumPayload(PackagePayload):
                      iutil.getSysroot()+_yum_target_langpack_conf)
 
     def _writeYumConfig(self, cache_dir=_yum_cache_dir):
-        """ Write out anaconda's main yum configuration file. """
+        """Write out anaconda's main yum configuration file."""
         buf = """
 [main]
 cachedir=%s
@@ -319,11 +335,11 @@ reposdir=%s
             self._yum.conf.cachedir = self._yum.conf.cachedir[len(root):]
 
     def _writeYumRepo(self, repo, repo_path):
-        """ Write a repo object to a yum repo.conf file
+        """Write a repo object to a yum repo.conf file
 
-            :param repo: Yum repository object
-            :param string repo_path: Path to write the repo to
-            :raises: PayloadSetupError if the repo doesn't have a url
+        :param repo: Yum repository object
+        :param string repo_path: Path to write the repo to
+        :raises: PayloadSetupError if the repo doesn't have a url
         """
         with open(repo_path, "w") as f:
             f.write("[%s]\n" % repo.id)
@@ -381,13 +397,13 @@ reposdir=%s
                 f.write("enablegroups=0\n")
 
     def _writeInstallConfig(self):
-        """ Write out the yum config that will be used to install packages.
+        """Write out the yum config that will be used to install packages.
 
-            Write out repo config files for all enabled repos, then
-            create a new YumBase instance with the new filesystem tree as its
-            install root.
+        Write out repo config files for all enabled repos, then
+        create a new YumBase instance with the new filesystem tree as its
+        install root.
 
-            This needs to be called from inside a yum_lock
+        This needs to be called from inside a yum_lock.
         """
         self._repos_dir = "/tmp/yum.repos.d"
         if not os.path.isdir(self._repos_dir):
@@ -427,16 +443,30 @@ reposdir=%s
     #           knowledge of the yum internals or, better yet, some convenience
     #           functions for multi-threaded applications
     def release(self):
-        from yum.packageSack import MetaSack
+        """Release package sacks and close their connections to the database."""
         with _yum_lock:
-            log.debug("deleting package sacks")
-            if hasattr(self._yum, "_pkgSack"):
-                self._yum._pkgSack = None
+            # Check if the meta package sack was created.
+            if self._yum._pkgSack is None:
+                return
 
-            self._yum.repos.pkgSack = MetaSack()
+            try:
+                # Drop cached data in every sack.
+                self._yum.pkgSack.dropCachedData()
 
-            for repo in self._yum.repos.repos.values():
-                repo._sack = None
+                # Close every sack.
+                for sack in self._yum.pkgSack.sacks.values():
+                    try:
+                        sack.close()
+                        del sack
+                    except RepoError as error:
+                        log.warn("The payload sack could not be closed: %s", error)
+
+                # Close the meta sack.
+                del self._yum.pkgSack
+
+            except RepoError as error:
+                # Do not propagate the repo exceptions.
+                log.warn("The payload could not be released: %s", error)
 
     def deleteYumTS(self):
         with _yum_lock:
@@ -466,17 +496,17 @@ reposdir=%s
 
     @property
     def baseRepo(self):
-        """ Return the current base repo id
-            :returns: repo id or None
+        """Return the current base repo id.
 
-            Methods that change (or could change) the base_repo
-            need to be decorated with @refresh_base_repo
+        :returns: repo id or None
+
+        NOTE: Methods that change (or could change) the base_repo
+              need to be decorated with @refresh_base_repo.
         """
         return self._base_repo
 
     def _refreshBaseRepo(self):
-        """ Examine the enabled repos and update _base_repo
-        """
+        """Examine the enabled repos and update _base_repo."""
         with _yum_lock:
             for repo_name in BASE_REPO_NAMES:
                 if repo_name in self.repos and \
@@ -505,31 +535,39 @@ reposdir=%s
                   return False
 
     def getRepo(self, repo_id):
-        """ Return the yum repo object. """
+        """Return the yum repo object."""
         with _yum_lock:
             repo = self._yum.repos.getRepo(repo_id)
 
         return repo
 
     def isRepoEnabled(self, repo_id):
-        """ Return True if repo is enabled. """
+        """Return True if repo is enabled."""
         try:
             return self.getRepo(repo_id).enabled
         except RepoError:
             return super(YumPayload, self).isRepoEnabled(repo_id)
 
+    def verifyAvailableRepositories(self):
+        """Verify availability of repositories."""
+        for repo in self._repoMD_list:
+            if not repo.verifyRepoMD():
+                log.debug("Can't reach repo %s", repo.id)
+                return False
+        return True
+
     @refresh_base_repo()
     def updateBaseRepo(self, fallback=True, root=None, checkmount=True):
-        """ Update the base repo based on self.data.method.
+        """Update the base repo based on self.data.method.
 
-            - Tear down any previous base repo devices, symlinks, &c.
-            - Reset the YumBase instance.
-            - Try to convert the new method to a base repo.
-            - If that fails, we'll use whatever repos yum finds in the config.
-            - Set up addon repos.
-            - Filter out repos that don't make sense to have around.
-            - Get metadata for all enabled repos, disabling those for which the
-              retrieval fails.
+        - Tear down any previous base repo devices, symlinks, &c.
+        - Reset the YumBase instance.
+        - Try to convert the new method to a base repo.
+        - If that fails, we'll use whatever repos yum finds in the config.
+        - Set up addon repos.
+        - Filter out repos that don't make sense to have around.
+        - Get metadata for all enabled repos, disabling those for which the
+          retrieval fails.
         """
         log.info("configuring base repo")
 
@@ -711,7 +749,7 @@ reposdir=%s
 
     @refresh_base_repo()
     def _configureAddOnRepo(self, repo):
-        """ Configure a single ksdata repo. """
+        """Configure a single ksdata repo."""
         url = repo.baseurl
         if url and url.startswith("nfs://"):
             # Let the assignment throw ValueError for bad NFS urls from kickstart
@@ -760,17 +798,17 @@ reposdir=%s
             self.data.repo.dataList().append(ks_repo)
 
     def _getAddons(self, baseurl, proxy_url, sslverify):
-        """ Check the baseurl or mirrorlist for a repository, see if it has any
-            valid addon repos and if so, return a list of (repo name, repo URL).
+        """Check the baseurl or mirrorlist for a repository, see if it has any
+        valid addon repos and if so, return a list of (repo name, repo URL).
 
-            :param baseurl: url of the repo
-            :type baseurl: string
-            :param proxy_url: Full URL of optional proxy or ""
-            :type proxy_url: string
-            :param sslverify: True if SSL certificate should be varified
-            :type sslverify: bool
-            :returns: list of tuples of addons (id, name, url)
-            :rtype: list of tuples
+        :param baseurl: url of the repo
+        :type baseurl: string
+        :param proxy_url: Full URL of optional proxy or ""
+        :type proxy_url: string
+        :param sslverify: True if SSL certificate should be varified
+        :type sslverify: bool
+        :returns: list of tuples of addons (id, name, url)
+        :rtype: list of tuples
         """
         retval = []
 
@@ -789,14 +827,15 @@ reposdir=%s
             return retval
 
         section = "variant-%s" % variant
-        if c.has_section(section) and c.has_option(section, "variants"):
-            validAddons = c.get(section, "variants").split(",")
+        # rhbz#1437510 - .treeinfo format change
+        if c.has_section(section) and c.has_option(section, "addons"):
+            validAddons = c.get(section, "addons").split(",")
         else:
             return retval
         log.debug("Addons found: %s", validAddons)
 
         for addon in validAddons:
-            addonSection = "variant-%s" % addon
+            addonSection = "addon-%s" % addon
             if not c.has_section(addonSection) or c.get(addonSection, "type") != "addon" or not c.has_option(addonSection, "repository"):
                 continue
 
@@ -806,7 +845,7 @@ reposdir=%s
         return retval
 
     def _getRepoMetadata(self, yumrepo):
-        """ Retrieve repo metadata if we don't already have it. """
+        """Retrieve repo metadata if we don't already have it."""
         # And try to grab its metadata.  We do this here so it can be done
         # on a per-repo basis, so we can then get some finer grained error
         # handling and recovery.
@@ -828,14 +867,14 @@ reposdir=%s
                 log.error("failed to get groups for repo %s", yumrepo.id)
 
     def _replaceVars(self, url):
-        """ Replace url variables with their values
+        """Replace url variables with their values.
 
-            :param url: url string to do replacement on
-            :type url:  string
-            :returns:   string with variables substituted
-            :rtype:     string or None
+        :param url: url string to do replacement on
+        :type url:  string
+        :returns:   string with variables substituted
+        :rtype:     string or None
 
-            Currently supports $releasever and $basearch
+        Currently supports $releasever and $basearch.
         """
         if not url:
             return url
@@ -847,7 +886,7 @@ reposdir=%s
         return url
 
     def _addYumRepo(self, name, baseurl, mirrorlist=None, proxyurl=None, **kwargs):
-        """ Add a yum repo to the YumBase instance. """
+        """Add a yum repo to the YumBase instance."""
         needsAdding = True
 
         # First, delete any pre-existing repo with the same name.
@@ -907,7 +946,7 @@ reposdir=%s
 
     @refresh_base_repo(lambda s, r_id: r_id.name in BASE_REPO_NAMES)
     def addRepo(self, newrepo):
-        """ Add a ksdata repo. """
+        """Add a ksdata repo."""
         log.debug("adding new repo %s", newrepo.name)
         self._addYumRepo(newrepo.name, newrepo.baseurl, newrepo.mirrorlist, newrepo.proxy)   # FIXME: handle MetadataError
         super(YumPayload, self).addRepo(newrepo)
@@ -921,7 +960,7 @@ reposdir=%s
 
     @refresh_base_repo(lambda s, r_id: r_id in BASE_REPO_NAMES)
     def removeRepo(self, repo_id):
-        """ Remove a repo as specified by id. """
+        """Remove a repo as specified by id."""
         log.debug("removing repo %s", repo_id)
 
         # if this is an NFS repo, we'll want to unmount the NFS mount after
@@ -944,7 +983,7 @@ reposdir=%s
 
     @refresh_base_repo(lambda s, r_id: r_id in BASE_REPO_NAMES)
     def enableRepo(self, repo_id):
-        """ Enable a repo as specified by id. """
+        """Enable a repo as specified by id."""
         log.debug("enabling repo %s", repo_id)
         if repo_id in self.repos:
             with _yum_lock:
@@ -953,7 +992,7 @@ reposdir=%s
 
     @refresh_base_repo(lambda s, r_id: r_id in BASE_REPO_NAMES)
     def disableRepo(self, repo_id):
-        """ Disable a repo as specified by id. """
+        """Disable a repo as specified by id."""
         log.debug("disabling repo %s", repo_id)
         if repo_id in self.repos:
             with _yum_lock:
@@ -968,7 +1007,7 @@ reposdir=%s
     ###
     @property
     def environments(self):
-        """ List of environment ids. """
+        """List of environment ids."""
         environments = []
         yum_groups = self._yumGroups
         if yum_groups:
@@ -1021,7 +1060,7 @@ reposdir=%s
         return False
 
     def environmentDescription(self, environmentid):
-        """ Return name/description tuple for the environment specified by id. """
+        """Return name/description tuple for the environment specified by id."""
         groups = self._yumGroups
         if not groups:
             return (environmentid, environmentid)
@@ -1035,7 +1074,7 @@ reposdir=%s
             return (environment.ui_name, environment.ui_description)
 
     def environmentId(self, environment):
-        """ Return environment id for the environment specified by id or name."""
+        """Return environment id for the environment specified by id or name."""
         groups = self._yumGroups
         if not groups:
             return environment
@@ -1093,7 +1132,7 @@ reposdir=%s
     ###
     @property
     def _yumGroups(self):
-        """ yum.comps.Comps instance. """
+        """yum.comps.Comps instance."""
         if not self._setup:
             return []
 
@@ -1109,7 +1148,7 @@ reposdir=%s
 
     @property
     def groups(self):
-        """ List of group ids. """
+        """List of group ids."""
         groups = []
         yum_groups = self._yumGroups
         if yum_groups:
@@ -1136,7 +1175,7 @@ reposdir=%s
         return list(lang_groups)
 
     def groupDescription(self, groupid):
-        """ Return name/description tuple for the group specified by id. """
+        """Return name/description tuple for the group specified by id."""
         groups = self._yumGroups
         if not groups:
             return (groupid, groupid)
@@ -1149,6 +1188,25 @@ reposdir=%s
 
             return (group.ui_name, group.ui_description)
 
+    def groupId(self, group_name):
+        """Translate group name to group ID.
+
+        :param group_name: Valid identifier for group specification.
+        :returns: Group ID.
+        :raise NoSuchGroup: If group_name doesn't exists.
+        :raise PayloadError: When Yum's groups are not available.
+        """
+        groups = self._yumGroups
+        if not groups:
+            raise PayloadError("Yum's groups are not available")
+
+        with _yum_lock:
+            if not groups.has_group(group_name):
+                raise NoSuchGroup(group_name)
+
+            group = groups.return_group(group_name)
+            return group.groupid
+
     def _isGroupVisible(self, groupid):
         groups = self._yumGroups
         if not groups:
@@ -1221,8 +1279,8 @@ reposdir=%s
     def _selectYumPackage(self, pkgid, required=False):
         """Mark a package for installation.
 
-           pkgid - The name of a package to be installed.  This could include
-                   a version or architecture component.
+        pkgid - The name of a package to be installed.  This could include
+                a version or architecture component.
         """
         log.debug("select package %s", pkgid)
         with _yum_lock:
@@ -1234,8 +1292,8 @@ reposdir=%s
     def _deselectYumPackage(self, pkgid):
         """Mark a package to be excluded from installation.
 
-           pkgid - The name of a package to be excluded.  This could include
-                   a version or architecture component.
+        pkgid - The name of a package to be excluded.  This could include
+                a version or architecture component.
         """
         log.debug("deselect package %s", pkgid)
         with _yum_lock:
@@ -1246,7 +1304,7 @@ reposdir=%s
     ###
     @property
     def spaceRequired(self):
-        """ The total disk space (Size) required for the current selection. """
+        """The total disk space (Size) required for the current selection."""
         return self._space_required
 
     def calculateSpaceNeeds(self):
@@ -1296,9 +1354,9 @@ reposdir=%s
                 time.sleep(100000)
 
     def _applyYumSelections(self):
-        """ Apply the selections in ksdata to yum.
+        """Apply the selections in ksdata to yum.
 
-            This follows the same ordering/pattern as kickstart.py.
+        This follows the same ordering/pattern as kickstart.py.
         """
         if self.data.packages.nocore:
             log.info("skipping core group due to %%packages --nocore; system may not be complete")
@@ -1383,6 +1441,9 @@ reposdir=%s
                 for msg in msgs:
                     log.warning(msg)
 
+                # Release the package sacks.
+                self.release()
+
                 raise DependencyError(msgs)
 
         self.calculateSpaceNeeds()
@@ -1390,6 +1451,9 @@ reposdir=%s
             log.info("%d packages selected totalling %s",
                      len(self._yum.tsInfo.getMembers()), self.spaceRequired)
 
+        # Release the package sacks.
+        self.release()
+
     def _select_kernel_package(self):
         kernels = self.kernelPackages
         selected = None
@@ -1430,7 +1494,7 @@ reposdir=%s
             self._handleMissing(e)
 
     def preInstall(self, packages=None, groups=None):
-        """ Perform pre-installation tasks. """
+        """Perform pre-installation tasks."""
         super(YumPayload, self).preInstall(packages, groups)
         progressQ.send_message(_("Starting package installation process"))
 
@@ -1456,15 +1520,14 @@ reposdir=%s
                 while True:
                     time.sleep(100000)
 
-
     def install(self):
-        """ Install the payload.
+        """Install the payload.
 
-            This writes out the yum transaction and then uses a Process thread
-            to execute it in a totally separate process.
+        This writes out the yum transaction and then uses a Process thread
+        to execute it in a totally separate process.
 
-            It monitors the status of the install and logs debug info, updates
-            the progress meter and cleans up when it is done.
+        It monitors the status of the install and logs debug info, updates
+        the progress meter and cleans up when it is done.
         """
         progress_map = {
             "PROGRESS_PREP"    : _("Preparing transaction from installation source"),
@@ -1523,7 +1586,7 @@ reposdir=%s
             exn = PayloadInstallError(str(e))
             if errorHandler.cb(exn) == ERROR_RAISE:
                 progressQ.send_quit(1)
-                iutil.ipmi_report(IPMI_ABORTED)
+                ipmi_abort(scripts=self.data.scripts)
                 sys.exit(1)
         finally:
             # log the contents of the scriptlet logfile if any
@@ -1543,7 +1606,7 @@ reposdir=%s
             exn = PayloadInstallError("\n".join(install_errors))
             if errorHandler.cb(exn) == ERROR_RAISE:
                 progressQ.send_quit(1)
-                iutil.ipmi_report(IPMI_ABORTED)
+                ipmi_abort(scripts=self.data.scripts)
                 sys.exit(1)
 
     def writeMultiLibConfig(self):
@@ -1579,7 +1642,7 @@ reposdir=%s
             log.error("failed to write out yum.conf: %s", e)
 
     def postInstall(self):
-        """ Perform post-installation tasks. """
+        """Perform post-installation tasks."""
         with _yum_lock:
             # clean up repo tmpdirs
             self._yum.cleanPackages()
@@ -1620,3 +1683,69 @@ reposdir=%s
         # Make sure yum is really done and gone and lets go of the yum.log
         self._yum.close()
         del self._yum
+
+class RepoMDMetaHash(object):
+    """Class that holds hash of a repomd.xml file content from a repository.
+    This class is used to test availability of this repository by comparing hashes.
+    """
+    def __init__(self, payload, repo):
+        self._repoId = repo.id
+        self._method = payload.data.method.method
+        self._urls = repo.urls
+        self._repomd_hash = ""
+
+    @property
+    def repoMDHash(self):
+        """Return MD5 hash of the repomd.xml file stored."""
+        return self._repomd_hash
+
+    @property
+    def id(self):
+        """Name of the repository."""
+        return self._repoId
+
+    def storeRepoMDHash(self):
+        """Download and store hash of the repomd.xml file content."""
+        repomd = self._downloadRepoMD(self._method)
+        self._repomd_hash = self._calculateHash(repomd)
+
+    def verifyRepoMD(self):
+        """Download and compare with stored repomd.xml file."""
+        new_repomd = self._downloadRepoMD(self._method)
+        new_repomd_hash = self._calculateHash(new_repomd)
+        return new_repomd_hash == self._repomd_hash
+
+    def _calculateHash(self, data):
+        m = hashlib.md5()
+        m.update(data)
+        return m.digest()
+
+    def _downloadRepoMD(self, method):
+        ugopts = {"ssl_verify_peer": not flags.noverifyssl,
+                  "ssl_verify_host": not flags.noverifyssl}
+        proxies = {}
+        repomd = ""
+
+        if hasattr(method, "proxy"):
+            proxy_url = method.proxy
+            try:
+                proxy = ProxyString(proxy_url)
+                proxies = {"http": proxy.url,
+                           "https": proxy.url}
+            except ProxyStringError as e:
+                log.info("Failed to parse proxy for test if repo available %s: %s",
+                         proxy_url, e)
+
+        ug = URLGrabber()
+
+        # Test all urls for this repo. If any of these is working it is enough.
+        for url in self._urls:
+            try:
+                repomd = ug.urlread("%s/repodata/repomd.xml" % url,
+                                    limit=50000, copy_local=True,
+                                    proxies=proxies, **ugopts)
+                break
+            except URLGrabError as e:
+                log.debug("Can't download new repomd.xml from %s. Error: %s", url, e)
+
+        return repomd
