@@ -30,6 +30,8 @@ from abc import ABCMeta, abstractproperty
 
 from pyanaconda.constants import ANACONDA_ENVIRON, FIRSTBOOT_ENVIRON
 from pyanaconda.errors import RemovedModuleError
+from pyanaconda import lifecycle
+from pyanaconda import screen_access
 from pykickstart.constants import FIRSTBOOT_RECONFIG, DISPLAY_MODE_TEXT
 
 import logging
@@ -224,6 +226,10 @@ class Spoke(object):
 
         self.visitedSinceApplied = True
 
+        # lists of callbacks to be called when the spoke is entered/exited by the user
+        self._entry_callbacks = [self.entry_logger, self._mark_screen_visited]
+        self._exit_callbacks = [self.exit_logger]
+
     @abstractproperty
     def data(self):
         pass
@@ -334,21 +340,110 @@ class Spoke(object):
         """
         raise NotImplementedError
 
-    def entry_logger(self):
+    def entry(self):
+        """Called once the spoke is about to be displayed.
+
+        Once called all the callbacks specified in the entry_callbacks list
+        property will be called in the list order.
+        """
+        for callback in self.entry_callbacks:
+            callback(self)
+
+    @property
+    def entry_callbacks(self):
+        """List of callback to be called once the spoke is entered by the user.
+
+        Each callback is called with a single argument, the spoke instance.
+        """
+        return self._entry_callbacks
+
+    def _mark_screen_visited(self, spoke_instance):
+        """Report the spoke screen as visited to the Spoke Access Manager."""
+        screen_access.sam.mark_screen_visited(spoke_instance.__class__.__name__)
+
+    def exit(self):
+        """Called once the spoke is exited by the used.
+
+        Once called all the callbacks specified in the exit_callbacks list
+        property will be called in the list order.
+        """
+        for callback in self.exit_callbacks:
+            callback(self)
+
+    @property
+    def exit_callbacks(self):
+        """List of callback to be called once the spoke is exited by the user.
+
+        Each callback is called with a single argument, the spoke instance.
+        """
+        return self._exit_callbacks
+
+    def entry_logger(self, spoke_instance):
         """Log immediately before this spoke is about to be displayed on the
            screen.  Subclasses may override this method if they want to log
            more specific information, but an overridden method should finish
            by calling this method so the entry will be logged.
         """
-        log.debug("Entered spoke: %s", self.__class__.__name__)
+        log.debug("Entered spoke: %s", spoke_instance.__class__.__name__)
 
-    def exit_logger(self):
+    def exit_logger(self, spoke_instance):
         """Log when a user leaves the spoke.  Subclasses may override this
            method if they want to log more specific information, but an
            overridden method should finish by calling this method so the
            exit will be logged.
         """
-        log.debug("Left spoke: %s", self.__class__.__name__)
+        log.debug("Left spoke: %s", spoke_instance.__class__.__name__)
+
+    # Initialization controller related code
+    #
+    # - initialization_controller
+    # -> The controller for this spokes and all others on the given hub.
+    # -> The controller has the init_done signal that can be used to trigger
+    #    actions that should happen once all spokes on the given Hub have
+    #    finished initialization.
+    # -> If there is no Hub (standalone spoke) the property is None
+    #
+    # - initialize_start()
+    # -> Should be called when Spoke initialization is started.
+    # -> Needs to be called explicitly, if we called it for every spoke by default
+    #    then any spoke that does not call initialize_done() would prevent the
+    #    controller form ever triggering the init_done signal.
+    #
+    # - initialize_done()
+    # -> Must be called by every spoke that calls initialize_start() or else the init_done
+    #    signal will never be emitted.
+
+    @property
+    def initialization_controller(self):
+        # standalone spokes don't have a category
+        if self.category:
+            return lifecycle.get_controller_by_category(category_name=self.category.__name__)
+        else:
+            return None
+
+    def initialize_start(self):
+        # get the correct controller for this spoke
+        spoke_controller = self.initialization_controller
+        # check if there actually is a controller for this spoke, there might not be one
+        # if this is a standalone spoke
+        if spoke_controller:
+            spoke_controller.module_init_start(self)
+
+    def initialize_done(self):
+        # get the correct controller for this spoke
+        spoke_controller = self.initialization_controller
+        # check if there actually is a controller for this spoke, there might not be one
+        # if this is a standalone spoke
+        if spoke_controller:
+            spoke_controller.module_init_done(self)
+
+    def __repr__(self):
+        """Return the class name as representation.
+
+        Returning the class name should be enough the uniquely identify a spoke.
+        """
+        return self.__class__.__name__
+
 
 # Inherit abstract methods from Spoke
 # pylint: disable=abstract-method
@@ -495,6 +590,10 @@ class Hub(object):
         self.paths = {}
         self._spokes = {}
 
+        # lists of callbacks to be called when the hub is entered/exited by the user
+        self._entry_callbacks = [self.entry_logger]
+        self._exit_callbacks = [self.exit_logger]
+
     @abstractproperty
     def data(self):
         pass
@@ -508,7 +607,41 @@ class Hub(object):
            name format string, directory name)"""
         self.paths[path_id] = paths
 
-    def entry_logger(self):
+    def entry(self):
+        """Called once the hub is about to be displayed.
+
+        Once called all the callbacks specified in the entry_callbacks list
+        property will be called in the list order.
+        """
+        for callback in self.entry_callbacks:
+            callback(self)
+
+    @property
+    def entry_callbacks(self):
+        """List of callback to be called once the hub is entered by the user.
+
+        Each callback is called with a single argument, the hub instance.
+        """
+        return self._entry_callbacks
+
+    def exit(self):
+        """Called once the hub is exited by the used.
+
+        Once called all the callbacks specified in the exit_callbacks list
+        property will be called in the list order.
+        """
+        for callback in self.exit_callbacks:
+            callback(self)
+
+    @property
+    def exit_callbacks(self):
+        """List of callback to be called once the hub is exited by the user.
+
+        Each callback is called with a single argument, the hub instance.
+        """
+        return self._exit_callbacks
+
+    def entry_logger(self, hub_instance):
         """Log immediately before this hub is about to be displayed on the
            screen.  Subclasses may override this method if they want to log
            more specific information, but an overridden method should finish
@@ -519,7 +652,7 @@ class Hub(object):
            and then coming back to the hub does not count as exiting and
            entering.
         """
-        log.debug("Entered hub: %s", self.__class__.__name__)
+        log.debug("Entered hub: %s", hub_instance.__class__.__name__)
 
     def _collectCategoriesAndSpokes(self):
         """This method is provided so that is can be overridden in a subclass
@@ -528,7 +661,7 @@ class Hub(object):
         """
         return collectCategoriesAndSpokes(self.paths, self.__class__, self.data.displaymode.displayMode)
 
-    def exit_logger(self):
+    def exit_logger(self, hub_instance):
         """Log when a user leaves the hub.  Subclasses may override this
            method if they want to log more specific information, but an
            overridden method should finish by calling this method so the
@@ -538,7 +671,7 @@ class Hub(object):
            user selects a spoke from the hub.  They are only exited when the
            continue or quit button is clicked on the hub.
         """
-        log.debug("Left hub: %s", self.__class__.__name__)
+        log.debug("Left hub: %s", hub_instance.__class__.__name__)
 
 def collect(module_pattern, path, pred):
     """Traverse the directory (given by path), import all files as a module
@@ -651,6 +784,7 @@ def collect(module_pattern, path, pred):
             continue
 
         except ImportError as imperr:
+            # pylint: disable=unsupported-membership-test
             if module_path and "pyanaconda" in module_path:
                 # failure when importing our own module:
                 raise
@@ -659,7 +793,9 @@ def collect(module_pattern, path, pred):
         finally:
             imp.release_lock()
 
+            # pylint: disable=unsubscriptable-object
             if mod_info and mod_info[0]:
+                # pylint: disable=unsubscriptable-object
                 mod_info[0].close()
 
         p = lambda obj: inspect.isclass(obj) and pred(obj)
@@ -690,8 +826,18 @@ def collect_spokes(mask_paths, category):
     """
     spokes = []
     for mask, path in mask_paths:
-        spokes.extend(collect(mask, path,
-                      lambda obj: hasattr(obj, "category") and obj.category is not None and obj.category.__name__ == category))
+        candidate_spokes = (collect(mask, path,
+                            lambda obj: hasattr(obj, "category") and obj.category is not None and obj.category.__name__ == category))
+        # filter out any spokes from the candidates that have already been visited by the user before
+        # (eq. before Anaconda or Initial Setup started) and should not be visible again
+        visible_spokes = []
+        for candidate in candidate_spokes:
+            if screen_access.sam.get_screen_visited(candidate.__name__):
+                log.info("Spoke %s will not be displayed because it has already been visited before.",
+                         candidate.__name__)
+            else:
+                visible_spokes.append(candidate)
+        spokes.extend(visible_spokes)
 
     return spokes
 
@@ -729,4 +875,16 @@ def collectCategoriesAndSpokes(paths, klass, displaymode):
     for c in categories:
         ret[c] = collect_spokes(paths["spokes"], c.__name__)
 
+    # As we now have a list of all categories this hub holds we can now register it's controller.
+    # We need the list of categories so that spokes can find out which controller they should use
+    # based on their category.
+    category_names = set()
+    for c in categories:
+        category_names.add(c.__name__)
+
+    # We have gathered all known category names and more are not expected to be added,
+    # so we can now add an initialization controller, which needs the final list
+    # of categories for the given hub.
+    lifecycle.add_controller(klass.__name__, category_names)
+
     return ret
