@@ -160,7 +160,7 @@ assert_exit_code() {
   local args="${1}"
   local ret="${2}"
 
-  ${path}firewall-cmd ${args} > /dev/null 2>&1
+  ${path}firewall-offline-cmd ${args} > /dev/null 2>&1
   got=$?
   if [[ "$got" -eq "$ret" ]]; then
     echo "${args} ... OK"
@@ -332,6 +332,15 @@ assert_good " --query-port=111-222/udp --zone=${default_zone}"
 assert_good "--remove-port 111-222/udp"
 assert_bad  " --query-port=111-222/udp"
 
+assert_good "   --add-port=5000/sctp"
+assert_good " --query-port=5000/sctp --zone=${default_zone}"
+assert_good "--remove-port 5000/sctp"
+assert_bad  " --query-port=5000/sctp"
+assert_good "   --add-port=222/dccp"
+assert_good " --query-port=222/dccp --zone=${default_zone}"
+assert_good "--remove-port 222/dccp"
+assert_bad  " --query-port=222/dccp"
+
 assert_good "   --add-port=80/tcp --add-port 443-444/udp"
 assert_good " --query-port=80/tcp --zone=${default_zone}"
 assert_good " --query-port=443-444/udp"
@@ -409,6 +418,10 @@ assert_good "   --add-forward-port=port=55:proto=tcp:toport=66:toaddr=7.7.7.7"
 assert_good " --query-forward-port port=55:proto=tcp:toport=66:toaddr=7.7.7.7 --zone=${default_zone}"
 assert_good "--remove-forward-port=port=55:proto=tcp:toport=66:toaddr=7.7.7.7"
 assert_bad  " --query-forward-port=port=55:proto=tcp:toport=66:toaddr=7.7.7.7"
+assert_good "   --add-forward-port=port=66:proto=sctp:toport=66:toaddr=7.7.7.7"
+assert_good " --query-forward-port port=66:proto=sctp:toport=66:toaddr=7.7.7.7 --zone=${default_zone}"
+assert_good "--remove-forward-port=port=66:proto=sctp:toport=66:toaddr=7.7.7.7"
+assert_bad  " --query-forward-port=port=66:proto=sctp:toport=66:toaddr=7.7.7.7"
 
 assert_good "   --add-forward-port=port=88:proto=udp:toport=99 --add-forward-port port=100:proto=tcp:toport=200"
 assert_good " --query-forward-port=port=100:proto=tcp:toport=200"
@@ -469,13 +482,12 @@ assert_good "--service=${myservice} --query-source-port=111-222/udp"
 assert_good "--service=${myservice} --remove-source-port 111-222/udp"
 assert_bad  "--service=${myservice} --query-source-port=111-222/udp"
 
-assert_bad  "--service=${myservice} --add-module=foo" # no valid helper
-assert_good "--service=${myservice} --add-module=nf_conntrack_sip"
-assert_good "--service=${myservice} --remove-module=nf_conntrack_sip"
-assert_good "--service=${myservice} --add-module=nf_conntrack_ftp"
-assert_good "--service=${myservice} --query-module=nf_conntrack_ftp"
-assert_good "--service=${myservice} --remove-module=nf_conntrack_ftp"
-assert_bad "--service=${myservice} --query-module=nf_conntrack_ftp"
+assert_good "--service=${myservice} --add-module=sip"
+assert_good "--service=${myservice} --remove-module=sip"
+assert_good "--service=${myservice} --add-module=ftp"
+assert_good "--service=${myservice} --query-module=ftp"
+assert_good "--service=${myservice} --remove-module=ftp"
+assert_bad "--service=${myservice} --query-module=ftp"
 
 assert_bad  "--service=${myservice} --set-destination=ipv4" # no address
 assert_bad  "--service=${myservice} --set-destination=ipv4:foo" # bad address
@@ -495,6 +507,18 @@ assert_good "--icmptype=${myicmp} --query-destination=ipv4"
 assert_good "--icmptype=${myicmp} --remove-destination=ipv4"
 assert_bad "--icmptype=${myicmp} --query-destination=ipv4"
 
+# test sctp and dccp ports
+assert_good "--service=${myservice} --add-port=666/sctp"
+assert_good "--service=${myservice} --remove-port=666/sctp"
+assert_good "--service=${myservice} --remove-port 666/sctp"
+assert_bad  "--service=${myservice} --query-port=666/sctp"
+assert_good "--service=${myservice} --add-port=999/dccp"
+assert_good "--service=${myservice} --remove-port=999/dccp"
+assert_good "--service=${myservice} --remove-port 999/dccp"
+assert_bad  "--service=${myservice} --query-port=999/dccp"
+assert_good "--service=${myservice} --add-port=666/sctp"
+assert_good "--service=${myservice} --add-port=999/dccp"
+
 # add them to zone
 assert_good "--zone=${myzone} --add-service=${myservice}"
 assert_good "--zone=${myzone} --add-icmp-block=${myicmp}"
@@ -529,6 +553,27 @@ assert_good "--zone=${zone} --remove-source=${source}"
 
 assert_good "--delete-ipset=${ipset}"
 
+# helper tests
+myhelper="myhelper"
+assert_bad "--new-helper=${myhelper} --module=foo"
+assert_good "--new-helper=${myhelper} --module=nf_conntrack_foo"
+assert_good_contains "--get-helpers" "${myhelper}"
+assert_good_empty "--helper=${myhelper} --get-family"
+assert_bad "--helper=${myhelper} --set-family=ipv5"
+assert_good "--helper=${myhelper} --set-family=ipv4"
+assert_good_equals "--helper=${myhelper} --get-family" "ipv4"
+assert_good "--helper=${myhelper} --set-family="
+assert_good_empty "--helper=${myhelper} --get-family"
+assert_good_empty "--helper=${myhelper} --get-ports"
+assert_good "--helper=${myhelper} --add-port=44/tcp"
+assert_good_notempty "--helper=${myhelper} --get-ports"
+assert_good "--helper=${myhelper} --query-port=44/tcp"
+assert_good "--helper=${myhelper} --remove-port=44/tcp"
+assert_bad "--helper=${myhelper} --query-port=44/tcp"
+assert_good_empty "--helper=${myhelper} --get-ports"
+assert_good "--delete-helper=${myhelper}"
+assert_bad_contains "--get-helpers" "${myhelper}"
+
 # exit return value tests
 assert_exit_code "--remove-port 122/udp" 0
 assert_exit_code "--add-port 122/udpp" 103
@@ -668,7 +713,9 @@ good_rules=(
  'rule family="ipv6" source address="1:2:3:4:6::" icmp-block name="redirect" log prefix="redirect" level="info" limit value="4/m"'
  'rule family="ipv4" source address="192.168.1.0/24" masquerade'
  'rule family="ipv6" masquerade'
- 'rule forward-port port="2222" to-port="22" to-addr="192.168.100.2" protocol="tcp" family="ipv4" source address="192.168.2.100"')
+ 'rule forward-port port="2222" to-port="22" to-addr="192.168.100.2" protocol="tcp" family="ipv4" source address="192.168.2.100"'
+ 'rule forward-port port="66" to-port="666" to-addr="192.168.100.2" protocol="sctp" family="ipv4" source address="192.168.2.100"'
+ 'rule forward-port port="99" to-port="999" to-addr="1::2:3:4:7" protocol="dccp" family="ipv6" source address="1:2:3:4:6::"')
 
 for (( i=0;i<${#good_rules[@]};i++)); do
   rule=${good_rules[${i}]}
